﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;

namespace DoodleGrouper.Classes
{
    /// <summary>
    /// Builds on the <see cref="ObservableCollection{T}"/> to provide a built in search option.
    /// </summary>
    /// <typeparam name="T">The type of the collection.</typeparam>
    public class SortableObservableCollection<T> : ObservableCollection<T>
    {
        private ListSortDirection _defaultDirection = ListSortDirection.Ascending;

        /// <summary>
        /// Initializes a new instance of the <see cref="SortableObservableCollection&lt;T&gt;"/> class.
        /// </summary>
        public SortableObservableCollection()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SortableObservableCollection&lt;T&gt;"/> class with an existing list.
        /// </summary>
        /// <param name="list">The list.</param>
        public SortableObservableCollection(List<T> list) : base(list)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SortableObservableCollection&lt;T&gt;"/> class with an existing instance of <see cref="IEnumerable{T}"/>.
        /// </summary>
        /// <param name="collection">The collection.</param>
        public SortableObservableCollection(IEnumerable<T> collection) : base(collection)
        {
        }

        /// <summary>
        /// Gets or sets the default direction for sorting the collection.
        /// </summary>
        /// <value>The default direction (defaults to <see cref="ListSortDirection.Ascending"/>).</value>
        public ListSortDirection DefaultDirection
        {
            get { return _defaultDirection; }
            set { _defaultDirection = value; }
        }

        /// <overloads>
        /// Sorts the collection. This method has 3 overloads.
        /// </overloads>
        /// <summary>
        /// Sorts the collection using the specified lambda expression.
        /// </summary>
        /// <typeparam name="TKey">The type of the key.</typeparam>
        /// <param name="keySelector">The key selector.</param>
        public void Sort<TKey>(Func<T, TKey> keySelector)
        {
            Sort(keySelector, _defaultDirection);
        }

        /// <summary>
        /// Sorts the collection using the specified lambda expression.
        /// </summary>
        /// <typeparam name="TKey">The type of the key.</typeparam>
        /// <param name="keySelector">The key selector.</param>
        /// <param name="direction">The direction to sort in.</param>
        public void Sort<TKey>(Func<T, TKey> keySelector, ListSortDirection direction)
        {
            switch (direction)
            {
                case ListSortDirection.Ascending:
                    {
                        applySort(Items.OrderBy(keySelector));
                        break;
                    }
                case ListSortDirection.Descending:
                    {
                        applySort(Items.OrderByDescending(keySelector));
                        break;
                    }
            }
        }

        /// <summary>
        /// Sorts the collection using the specified lambda expression.
        /// </summary>
        /// <typeparam name="TKey">The type of the key.</typeparam>
        /// <param name="keySelector">The key selector.</param>
        /// <param name="comparer">The comparer.</param>
        public void Sort<TKey>(Func<T, TKey> keySelector, IComparer<TKey> comparer)
        {
            applySort(Items.OrderBy(keySelector, comparer));
        }

        private void applySort(IEnumerable<T> sortedItems)
        {
            List<T> sortedItemsList = sortedItems.ToList();
            int newIndex = 0;

            foreach (T item in sortedItemsList)
            {
                int oldIndex = IndexOf(item);

                if (oldIndex != newIndex)
                {
                    Insert(newIndex, item);

                    RemoveAt(oldIndex + 1);
                }
                newIndex++;
            }
        }
    }
}