﻿using System;
using System.Collections.Specialized;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using DoodleGrouper.Classes;
using Seesmic.Sdp.Extensibility;

namespace DoodleGrouper
{
    /// <summary>
    /// This plug-in allows you to group users from different accounts for filtering, sorting, etc. 
    /// </summary>
    [Export(typeof (IPlugin))]
    public class Plugin : IPlugin
    {
        private static SortableObservableCollection<UserList> _userLists;
        private static IShellService _shell;
        private static INotificationService _notify;
        private static ILogService _log;
        private static IStorageService _storage;

        private static readonly ResourceDictionary _Templates;
        private static readonly Guid _AccountProviderId = new Guid("3671FC7F-570C-4428-AED3-18FB3FD6EB4B");

        static Plugin()
        {
            _Templates = new ResourceDictionary
                                {
                                    Source =
                                        new Uri("/DoodleGrouper;component/Templates/DataTemplates.xaml",
                                                UriKind.Relative)
                                };
        }

        [Import]
        public IShellService ShellServiceImport
        {
            set { _shell = value; }
        }

        [Import]
        public IStorageService StorageServiceImport
        {
            set { _storage = value; }
        }

        [Import]
        public ILogService LogServiceImport
        {
            set { _log = value; }
        }

        [Import]
        public INotificationService NotificationServiceImport
        {
            set { _notify = value; }
        }

        /// <summary>
        /// Gets the shell service.
        /// </summary>
        /// <value>The shell service.</value>
        internal IShellService ShellService
        {
            get { return _shell; }
        }

        /// <summary>
        /// Gets the storage service.
        /// </summary>
        /// <value>The storage service.</value>
        internal static IStorageService StorageService
        {
            get { return _storage; }
        }

        /// <summary>
        /// Gets the log service.
        /// </summary>
        /// <value>The log service.</value>
        internal static ILogService LogService
        {
            get { return _log; }
        }

        /// <summary>
        /// Gets the notification service.
        /// </summary>
        /// <value>The notification service.</value>
        internal static INotificationService NotificationService
        {
            get { return _notify; }
        }

        /// <summary>
        /// Gets the user lists.
        /// </summary>
        /// <value>The user lists.</value>
        internal static SortableObservableCollection<UserList> UserLists
        {
            get { return _userLists; }
        }

        /// <summary>
        /// Gets the resources.
        /// </summary>
        /// <value>The resources.</value>
        internal static ResourceDictionary Resources
        {
            get { return _Templates; }
        }

        /// <summary>
        /// Gets the account provider id.
        /// </summary>
        /// <value>The account provider id.</value>
        public static Guid AccountProviderId { get { return _AccountProviderId; } }

        /// <summary>
        /// Initializes this instance.
        /// </summary>
        /// <remarks>
        /// Picks up the userlists from storage, loads them up and creates new Sidebar Actions for each one. 
        /// </remarks>
        public void Initialize()
        {
            ITimelineItemAction addToGroup = new AddToGroup();
            ShellService.RegisterTimelineItemAction(addToGroup);

            LogInfo("Loading user lists.");
            _userLists = StorageService.GetValue<SortableObservableCollection<UserList>>(Id, "UserLists");

            if (null == _userLists)
            {
                LogInfo("No saved user lists.");
                _userLists = new SortableObservableCollection<UserList>();
            }
            else
            {
                LogInfo("User lists loaded from storage.");

                _userLists.Sort(u => u.Name);

                buildUserLists();
            }

            UserLists.CollectionChanged += updateSidebar;
        }

        /// <summary>
        /// Commits the settings.
        /// </summary>
        /// <remarks>
        /// The plugin currently doesn't have any settings to save beyond the user lists, and these are saved as we go.
        /// </remarks>
        public void CommitSettings()
        {
            // TODO: Save settings from UI.
        }

        /// <summary>
        /// Reverts the settings.
        /// </summary>
        /// <remarks>
        /// Nothing to revert here.
        /// </remarks>
        public void RevertSettings()
        {
            // TODO: Revert settings from UI.
        }

        /// <summary>
        /// Gets the id.
        /// </summary>
        /// <value>The id.</value>
        public Guid Id
        {
            get { return _AccountProviderId; }
        }

        /// <summary>
        /// Gets the settings template.
        /// </summary>
        /// <value>The settings template.</value>
        /// <remarks>There is no settings template at the moment.</remarks>
        public DataTemplate SettingsTemplate
        {
            get { return null; }
        }

        /// <summary>
        /// Updates the sidebar with changes to the <see cref="UserList"/> collection.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Collections.Specialized.NotifyCollectionChangedEventArgs"/> instance containing the event data.</param>
        private void updateSidebar(object sender, NotifyCollectionChangedEventArgs e)
        {
            foreach (UserList list in from object newItem in e.NewItems select newItem as UserList)
            {
                ShellService.RegisterSidebarAction(SidebarActionCategory.Userlists, list);
            }
        }

        /// <summary>
        /// Writes an Info message to the log.
        /// </summary>
        /// <param name="message">The message.</param>
        internal static void LogInfo(string message)
        {
            LogService.Info(string.Format("Plug-in: DoodleGrouper: {0}", message));
        }

        /// <summary>
        /// Builds the Side Bar items from the <see cref="UserList"/>.
        /// </summary>
        private void buildUserLists()
        {
            foreach (UserList userList in _userLists)
            {
                ShellService.RegisterSidebarAction(SidebarActionCategory.Userlists, userList);
            }
        }

        /// <summary>
        /// Saves the user lists.
        /// </summary>
        public static void SaveUserLists()
        {
            LogInfo("Saving out user list data.");
            StorageService.SetValue(AccountProviderId, "UserLists", UserLists);
            NotificationService.ShowBarNotification(new BarNotificationInfo { AutoDismiss = true, Kind = BarNotificationKind.Success, QueueTimeout = new TimeSpan(0,0,3), Text = " User lists saved." });
        }
    }
}